.form-signin input[type="text"] {
	margin-bottom: -1px;
	border-bottom-left-radius: 0;
	border-bottom-right-radius: 0;
}

.form-signin input[type="password"] {
	margin-bottom: 10px;
	border-top-left-radius: 0;
	border-top-right-radius: 0;
}

.form-signin a,
.form-signin a:visited,
.form-signin a:active {
	color: inherit !important;
}

.brand-icon {
	vertical-align: bottom;
}

.navbar-brand > img {
    display: inline-block;
}

.login-page {
	max-width: 300px;
}

.lazyload {
	background: #C3C3C3;
}

.divider {
	height:50px;
	margin:0 9px;
	border-left:1px solid #e1e1e1;
}

.vertical-space-small {
	height: 50px;
}

.multipasteQueue .items>div {
	height: 190px;
	display: inline-block;
	width: 150px;
	vertical-align: top;
	margin: 0 1px 20px 1px;
}

.multipasteQueue .items .item {
	height: 160px;
}

.nav li.multipaste_button a {
	color: #fff;
}

.ajaxFeedback {
	display: inline-block;
}

/* Source: http://stackoverflow.com/a/26283602 */
.glyphicon.spinning {
	animation: spin 1s infinite linear;
	-webkit-animation: spin2 1s infinite linear;
}

@keyframes spin {
	from { transform: scale(1) rotate(0deg); }
	to { transform: scale(1) rotate(360deg); }
}

@-webkit-keyframes spin2 {
	from { -webkit-transform: rotate(0deg); }
	to { -webkit-transform: rotate(360deg); }
}

@media (max-width: 768px) {
	.dont-float {
		float: left;
		margin-top: 0;
		margin-bottom: 0;
	}
	.dont-float > li {
		float: left;
	}
	.dont-float > li > a {
		padding-top: 15px;
		padding-bottom: 15px;
	}
	.dont-float:first-child {
		margin-left: 0px;
	}
	.dont-float.navbar-right:last-child {
		margin-right: -15px;
	}
}

.nav-history {
	height: 63px;
}

.nav-history .container {
	padding-top: 20px;
	margin-bottom: 1em;
	position: fixed;
	background: #fff;
	left: 0;
	right: 0;
	margin-top: -19px;
}

.navbar-paste {
	margin-bottom: 0;
}

.inline-input {
	width: 200px;
}

.file-upload {
	width: 100%;
}

.navbar-nav > li > .dropdown-menu {
	margin-top: 2px;
}

.anchor {
	visibility: hidden;
	padding-top: 20px;
	margin-top: -20px;
}

@media (max-width: 767px) {
	.navbar-fixed-top {
		position: relative;
	}
	.nav-history .container {
		position: relative;
	}
	.nav-history {
		height: auto;
	}
}

@media (min-width: 768px) {
	.anchor {
		visibility: hidden;
		padding-top: 70px;
		margin-top: -70px;
	}

	#navbar-height {
		height: 70px;
	}
}

textarea.text-upload {
	height: 200px;
}

@media (min-height: 700px) {
	textarea.text-upload {
		height: 300px;
	}
}

@media (min-height: 800px) {
	textarea.text-upload {
		height: 400px;
	}
}

.error h1 {
	font-size: 19px;
}

.nowrap {
	white-space: nowrap;
}

.wrap {
	word-break: break-word;
	word-wrap: normal;
}

.alert-wide {
	text-align: center;
	border-radius: 0;
}

code, pre, textarea {
	font-family: "Dejavu sans mono", Monaco, monospace;
}

.content {
	width: 100%;
	background: #fff;
}

/* Sticky footer styles
 * Source: http://twitter.github.io/bootstrap/examples/sticky-footer.html
 -------------------------------------------------- */
.footer {
	background-color: #f5f5f5;
	border-top: 1px solid #e5e5e5;
	/*padding: 30px 0;*/
	/*margin-top: 70px;*/
	text-align: center;
}


html,
body {
	height: 100%;
	/* The html and body elements cannot have any padding or margin. */
}

/* Wrapper for page content to push down footer */
#wrap {
	min-height: 100%;
	height: auto !important;
	height: 100%;
	/* Negative indent footer by it's height */
	margin: 0 auto -83px;
}

/* Set the fixed height of the footer here */
#footer {
	height: 83px;
	font-size: 12px;
	padding: 15px 0;
}

#footer,
#footer li {
	line-height: 12px;
}

/* Set to footer height + 70px "padding" */
#push {
	height: 153px;
}

#footer p,
.footer-links {
	margin: 4px 0;
}
.footer-links li {
	display: inline;
	padding: 0 2px;
}
.footer-links li:first-child {
	padding-left: 0;
}

.container-wide {
	padding: 0;
	max-width: 100%;
	margin-left: 20px;
	margin-right: 20px;
	margin-bottom: 20px;
}

.simple-container {
	margin-left: 20px;
	margin-right: 20px;
	margin-bottom: 20px;
}

.paste-container {
	padding: 3px;
	background: #eee;
}

.code .code-container {
	counter-reset: line-numbering;
}

.code {display: table; margin: 0;}
.code .table-row {display: table-row;}
.code .table-cell {display: table-cell;}

.code .code-container a.linenumber {
	-webkit-touch-callout: none;
	-webkit-user-select: none;
	-khtml-user-select: none;
	-moz-user-select: none;
	-ms-user-select: none;
	user-select: none;
}

.code .code-container a.linenumber::before {
	content: counter(line-numbering);
	counter-increment: line-numbering;
}

.code .code-container a.linenumber {
	padding: 0 6px 0 16px;
	text-align: right;
	background: #ececec;
	border-right: 1px solid #ddd;
	text-align: right;
	margin-left: -2px;
	color: #aaa;
	word-break: normal;
}

.code .code-container :last-child a {
	padding-bottom: 10px;
}

.code .code-container :first-child a {
	padding-top: 10px;
}

.code .code-container .table-row .line {
	padding-left: 10px;
	width: 100%
}

.content .highlight > .code-container {
	background-color: transparent;
	border: 0;
	border-radius: 0;
	margin: 0;
	padding: 0;
	font-size: 12px;
	line-height: 1.4;
	white-space: pre-wrap;
	word-break: break-word;
	word-wrap: normal;
}
.code {
	width: 100%;
	padding-top: 0;
}
.code .highlight {
	position: relative;
	padding-left: 0;
}
.code .highlight .code-container {
	position: relative;
	display: table;
	color: inherit;
}
.highlight_line {
	background: #ffffcc;
}

.ui-autocomplete {
	z-index: 1500;
}

.popover {
	word-break: break-word;
	word-wrap: normal;
	max-width: 400px;
	min-width: 250px
}

#upload_history .popover {
    max-width: 100%;
}

#upload_history .popover pre {
    background-color: transparent;
    border: 0;
    margin: 0;
    line-height: 1.4;
}

.upload_thumbnails {
	margin: 0 auto;
	width: 152px;
}

/* min-width and width increase by 152px each step (one more image per line)
 * Take care to align to bootstrap's viewport breakpoints */
@media (min-width: 389px) {
	.upload_thumbnails {
		width: 304px;
	}
}

@media (min-width: 541px) {
	.upload_thumbnails {
		width: 456px;
	}
}

@media (min-width: 693px) {
	.upload_thumbnails {
		width: 608px;
	}
}

/* bootstrap viewport breakpoint @768px */
/* bootstrap viewport breakpoint @992px */
@media (min-width: 992px) {
	.upload_thumbnails {
		width: 760px;
	}
}

/* bootstrap viewport breakpoint @1220px */
@media (min-width: 1200px) {
	.upload_thumbnails {
		width: 1064px;
	}
}

/* XXX: change values above if img width chanegs */
.upload_thumbnails img.thumb,
.upload_thumbnails a {
	width: 150px;
	height: 150px;
}

.upload_thumbnails a {
	margin: 1px;
	display: inline-block;
}

.upload_thumbnails .marked {
	background: red;
}

.upload_thumbnails .marked img {
	opacity: 0.4;
}

.tabwidth-dropdown { padding: 15px; }
input[type="number"]::-webkit-outer-spin-button,
input[type="number"]::-webkit-inner-spin-button {
	-webkit-appearance: none;
	margin: 0;
}
input[type="number"] { 	-moz-appearance: textfield; }

video {
	max-width: 100%;
}

/* highlighting theme */
.hll { background-color: #ffffcc }
.c { color: #999988; font-style: italic } /* Comment */
.err { color: #a61717; background-color: #e3d2d2 } /* Error */
.k { color: #000000; font-weight: bold } /* Keyword */
.o { color: #000000; font-weight: bold } /* Operator */
.cm { color: #999988; font-style: italic } /* Comment.Multiline */
.cp { color: #999999; font-weight: bold; font-style: italic } /* Comment.Preproc */
.c1 { color: #999988; font-style: italic } /* Comment.Single */
.cs { color: #999999; font-weight: bold; font-style: italic } /* Comment.Special */
.gd { color: #000000; background-color: #ffdddd } /* Generic.Deleted */
.ge { color: #000000; font-style: italic } /* Generic.Emph */
.gr { color: #aa0000 } /* Generic.Error */
.gh { color: #999999 } /* Generic.Heading */
.gi { color: #000000; background-color: #ddffdd } /* Generic.Inserted */
.go { color: #888888 } /* Generic.Output */
.gp { color: #555555 } /* Generic.Prompt */
.gs { font-weight: bold } /* Generic.Strong */
.gu { color: #aaaaaa } /* Generic.Subheading */
.gt { color: #aa0000 } /* Generic.Traceback */
.kc { color: #000000; font-weight: bold } /* Keyword.Constant */
.kd { color: #000000; font-weight: bold } /* Keyword.Declaration */
.kn { color: #000000; font-weight: bold } /* Keyword.Namespace */
.kp { color: #000000; font-weight: bold } /* Keyword.Pseudo */
.kr { color: #000000; font-weight: bold } /* Keyword.Reserved */
.kt { color: #445588; font-weight: bold } /* Keyword.Type */
.m { color: #009999 } /* Literal.Number */
.s { color: #d01040 } /* Literal.String */
.na { color: #008080 } /* Name.Attribute */
.nb { color: #0086B3 } /* Name.Builtin */
.nc { color: #445588; font-weight: bold } /* Name.Class */
.no { color: #008080 } /* Name.Constant */
.nd { color: #3c5d5d; font-weight: bold } /* Name.Decorator */
.ni { color: #800080 } /* Name.Entity */
.ne { color: #990000; font-weight: bold } /* Name.Exception */
.nf { color: #990000; font-weight: bold } /* Name.Function */
.nl { color: #990000; font-weight: bold } /* Name.Label */
.nn { color: #555555 } /* Name.Namespace */
.nt { color: #000080 } /* Name.Tag */
.nv { color: #008080 } /* Name.Variable */
.ow { color: #000000; font-weight: bold } /* Operator.Word */
.w { color: #bbbbbb } /* Text.Whitespace */
.mf { color: #009999 } /* Literal.Number.Float */
.mh { color: #009999 } /* Literal.Number.Hex */
.mi { color: #009999 } /* Literal.Number.Integer */
.mo { color: #009999 } /* Literal.Number.Oct */
.sb { color: #d01040 } /* Literal.String.Backtick */
.sc { color: #d01040 } /* Literal.String.Char */
.sd { color: #d01040 } /* Literal.String.Doc */
.s2 { color: #d01040 } /* Literal.String.Double */
.se { color: #d01040 } /* Literal.String.Escape */
.sh { color: #d01040 } /* Literal.String.Heredoc */
.si { color: #d01040 } /* Literal.String.Interpol */
.sx { color: #d01040 } /* Literal.String.Other */
.sr { color: #009926 } /* Literal.String.Regex */
.s1 { color: #d01040 } /* Literal.String.Single */
.ss { color: #990073 } /* Literal.String.Symbol */
.bp { color: #999999 } /* Name.Builtin.Pseudo */
.vc { color: #008080 } /* Name.Variable.Class */
.vg { color: #008080 } /* Name.Variable.Global */
.vi { color: #008080 } /* Name.Variable.Instance */
.il { color: #009999 } /* Literal.Number.Integer.Long */

/* based on https://bitbucket.org/kevinburke/markdowncss/overview */
.markdownrender {
	padding: 0 12px;
}
.markdownrender h1, .markdownrender h2, .markdownrender h3, .markdownrender h4 {
	color: #111111;
	font-weight: 400;
}
.markdownrender h1, .markdownrender h2, .markdownrender h3, .markdownrender h4, .markdownrender p {
	margin-bottom: 24px;
	padding: 0;
}
.markdownrender h1 {
	font-size: 2.5em;
}
.markdownrender h2 {
	font-size: 2em;
	margin: 24px 0 6px;
}
.markdownrender h3 {
	font-size: 1.7em;
}
.markdownrender h4 {
	font-size: 1.5em;
}
.markdownrender h5 {
	font-size: 1.3em;
}
.markdownrender li {
	line-height: 1.5em;
}
.markdownrender li ul, .markdownrender li ul {
	margin-left: 24px;
}
.markdownrender p, .markdownrender ul, .markdownrender ol {
	font-size: 0.9em;
	line-height: 1.5em;
	max-width: 740px;
}
.markdownrender pre {
	padding: 0px 24px;
	max-width: 800px;
	white-space: pre-wrap;
}
.markdownrender code {
	font-family: Consolas, Monaco, Andale Mono, monospace;
	line-height: 1.2em;
	font-size: 0.85em;
}
.markdownrender aside {
	display: block;
	float: right;
	width: 390px;
}
.markdownrender blockquote {
	border-left: 0.5em solid #d6d6d6;
	padding: 0 1em;
	margin-left:0;
	max-width: 676px;
	background: #f6f6f6;
}
.markdownrender blockquote cite {
	font-size:0.85em;
	line-height:1.3em;
	color:#bfbfbf;
}
.markdownrender blockquote p {
	color: #666;
	max-width: 660px;
}
.markdownrender hr {
	width: 740px;
	text-align: left;
	margin: 0 auto 0 0;
	color: #999;
}

/* tables */
table.tablesorter thead tr th, table.tablesorter tfoot tr th {
	background-color: #e6EEEE;
	padding-right: 20px;
}
table.tablesorter thead tr .header {
	background-image: url(../img/bg.gif);
	background-repeat: no-repeat;
	background-position: center right;
	cursor: pointer;
}
table.tablesorter tbody td {
	color: #3D3D3D;
	vertical-align: top;
}
table.tablesorter thead tr .headerSortUp {
	background-image: url(../img/asc.gif);
}
table.tablesorter thead tr .headerSortDown {
	background-image: url(../img/desc.gif);
}
table.tablesorter thead tr .headerSortDown, table.tablesorter thead tr .headerSortUp {
	background-color: #8dbdd8;
}

/* ansi2html */
.ansi1 { font-weight: bold; }
.ansi3 { font-weight: italic; }
.ansi4 { text-decoration: underline; }
.ansi9 { text-decoration: line-through; }
.ansi30 { color: #000316; }
.ansi31 { color: #aa0000; }
.ansi32 { color: #00aa00; }
.ansi33 { color: #aa5500; }
.ansi34 { color: #0000aa; }
.ansi35 { color: #E850A8; }
.ansi36 { color: #00aaaa; }
.ansi37 { color: #F5F1DE; }
.ansi40 { background-color: #000316; }
.ansi41 { background-color: #aa0000; }
.ansi42 { background-color: #00aa00; }
.ansi43 { background-color: #aa5500; }
.ansi44 { background-color: #0000aa; }
.ansi45 { background-color: #E850A8; }
.ansi46 { background-color: #00aaaa; }
.ansi47 { background-color: #F5F1DE; }
.ansi38-16 { color: #000000; }
.ansi48-16 { background: #000000; }
.ansi38-17 { color: #00002a; }
.ansi48-17 { background: #00002a; }
.ansi38-18 { color: #000054; }
.ansi48-18 { background: #000054; }
.ansi38-19 { color: #00007e; }
.ansi48-19 { background: #00007e; }
.ansi38-20 { color: #0000a8; }
.ansi48-20 { background: #0000a8; }
.ansi38-21 { color: #0000d2; }
.ansi48-21 { background: #0000d2; }
.ansi38-52 { color: #2a0000; }
.ansi48-52 { background: #2a0000; }
.ansi38-53 { color: #2a002a; }
.ansi48-53 { background: #2a002a; }
.ansi38-54 { color: #2a0054; }
.ansi48-54 { background: #2a0054; }
.ansi38-55 { color: #2a007e; }
.ansi48-55 { background: #2a007e; }
.ansi38-56 { color: #2a00a8; }
.ansi48-56 { background: #2a00a8; }
.ansi38-57 { color: #2a00d2; }
.ansi48-57 { background: #2a00d2; }
.ansi38-88 { color: #540000; }
.ansi48-88 { background: #540000; }
.ansi38-89 { color: #54002a; }
.ansi48-89 { background: #54002a; }
.ansi38-90 { color: #540054; }
.ansi48-90 { background: #540054; }
.ansi38-91 { color: #54007e; }
.ansi48-91 { background: #54007e; }
.ansi38-92 { color: #5400a8; }
.ansi48-92 { background: #5400a8; }
.ansi38-93 { color: #5400d2; }
.ansi48-93 { background: #5400d2; }
.ansi38-124 { color: #7e0000; }
.ansi48-124 { background: #7e0000; }
.ansi38-125 { color: #7e002a; }
.ansi48-125 { background: #7e002a; }
.ansi38-126 { color: #7e0054; }
.ansi48-126 { background: #7e0054; }
.ansi38-127 { color: #7e007e; }
.ansi48-127 { background: #7e007e; }
.ansi38-128 { color: #7e00a8; }
.ansi48-128 { background: #7e00a8; }
.ansi38-129 { color: #7e00d2; }
.ansi48-129 { background: #7e00d2; }
.ansi38-160 { color: #a80000; }
.ansi48-160 { background: #a80000; }
.ansi38-161 { color: #a8002a; }
.ansi48-161 { background: #a8002a; }
.ansi38-162 { color: #a80054; }
.ansi48-162 { background: #a80054; }
.ansi38-163 { color: #a8007e; }
.ansi48-163 { background: #a8007e; }
.ansi38-164 { color: #a800a8; }
.ansi48-164 { background: #a800a8; }
.ansi38-165 { color: #a800d2; }
.ansi48-165 { background: #a800d2; }
.ansi38-196 { color: #d20000; }
.ansi48-196 { background: #d20000; }
.ansi38-197 { color: #d2002a; }
.ansi48-197 { background: #d2002a; }
.ansi38-198 { color: #d20054; }
.ansi48-198 { background: #d20054; }
.ansi38-199 { color: #d2007e; }
.ansi48-199 { background: #d2007e; }
.ansi38-200 { color: #d200a8; }
.ansi48-200 { background: #d200a8; }
.ansi38-201 { color: #d200d2; }
.ansi48-201 { background: #d200d2; }
.ansi38-22 { color: #002a00; }
.ansi48-22 { background: #002a00; }
.ansi38-23 { color: #002a2a; }
.ansi48-23 { background: #002a2a; }
.ansi38-24 { color: #002a54; }
.ansi48-24 { background: #002a54; }
.ansi38-25 { color: #002a7e; }
.ansi48-25 { background: #002a7e; }
.ansi38-26 { color: #002aa8; }
.ansi48-26 { background: #002aa8; }
.ansi38-27 { color: #002ad2; }
.ansi48-27 { background: #002ad2; }
.ansi38-58 { color: #2a2a00; }
.ansi48-58 { background: #2a2a00; }
.ansi38-59 { color: #2a2a2a; }
.ansi48-59 { background: #2a2a2a; }
.ansi38-60 { color: #2a2a54; }
.ansi48-60 { background: #2a2a54; }
.ansi38-61 { color: #2a2a7e; }
.ansi48-61 { background: #2a2a7e; }
.ansi38-62 { color: #2a2aa8; }
.ansi48-62 { background: #2a2aa8; }
.ansi38-63 { color: #2a2ad2; }
.ansi48-63 { background: #2a2ad2; }
.ansi38-94 { color: #542a00; }
.ansi48-94 { background: #542a00; }
.ansi38-95 { color: #542a2a; }
.ansi48-95 { background: #542a2a; }
.ansi38-96 { color: #542a54; }
.ansi48-96 { background: #542a54; }
.ansi38-97 { color: #542a7e; }
.ansi48-97 { background: #542a7e; }
.ansi38-98 { color: #542aa8; }
.ansi48-98 { background: #542aa8; }
.ansi38-99 { color: #542ad2; }
.ansi48-99 { background: #542ad2; }
.ansi38-130 { color: #7e2a00; }
.ansi48-130 { background: #7e2a00; }
.ansi38-131 { color: #7e2a2a; }
.ansi48-131 { background: #7e2a2a; }
.ansi38-132 { color: #7e2a54; }
.ansi48-132 { background: #7e2a54; }
.ansi38-133 { color: #7e2a7e; }
.ansi48-133 { background: #7e2a7e; }
.ansi38-134 { color: #7e2aa8; }
.ansi48-134 { background: #7e2aa8; }
.ansi38-135 { color: #7e2ad2; }
.ansi48-135 { background: #7e2ad2; }
.ansi38-166 { color: #a82a00; }
.ansi48-166 { background: #a82a00; }
.ansi38-167 { color: #a82a2a; }
.ansi48-167 { background: #a82a2a; }
.ansi38-168 { color: #a82a54; }
.ansi48-168 { background: #a82a54; }
.ansi38-169 { color: #a82a7e; }
.ansi48-169 { background: #a82a7e; }
.ansi38-170 { color: #a82aa8; }
.ansi48-170 { background: #a82aa8; }
.ansi38-171 { color: #a82ad2; }
.ansi48-171 { background: #a82ad2; }
.ansi38-202 { color: #d22a00; }
.ansi48-202 { background: #d22a00; }
.ansi38-203 { color: #d22a2a; }
.ansi48-203 { background: #d22a2a; }
.ansi38-204 { color: #d22a54; }
.ansi48-204 { background: #d22a54; }
.ansi38-205 { color: #d22a7e; }
.ansi48-205 { background: #d22a7e; }
.ansi38-206 { color: #d22aa8; }
.ansi48-206 { background: #d22aa8; }
.ansi38-207 { color: #d22ad2; }
.ansi48-207 { background: #d22ad2; }
.ansi38-28 { color: #005400; }
.ansi48-28 { background: #005400; }
.ansi38-29 { color: #00542a; }
.ansi48-29 { background: #00542a; }
.ansi38-30 { color: #005454; }
.ansi48-30 { background: #005454; }
.ansi38-31 { color: #00547e; }
.ansi48-31 { background: #00547e; }
.ansi38-32 { color: #0054a8; }
.ansi48-32 { background: #0054a8; }
.ansi38-33 { color: #0054d2; }
.ansi48-33 { background: #0054d2; }
.ansi38-64 { color: #2a5400; }
.ansi48-64 { background: #2a5400; }
.ansi38-65 { color: #2a542a; }
.ansi48-65 { background: #2a542a; }
.ansi38-66 { color: #2a5454; }
.ansi48-66 { background: #2a5454; }
.ansi38-67 { color: #2a547e; }
.ansi48-67 { background: #2a547e; }
.ansi38-68 { color: #2a54a8; }
.ansi48-68 { background: #2a54a8; }
.ansi38-69 { color: #2a54d2; }
.ansi48-69 { background: #2a54d2; }
.ansi38-100 { color: #545400; }
.ansi48-100 { background: #545400; }
.ansi38-101 { color: #54542a; }
.ansi48-101 { background: #54542a; }
.ansi38-102 { color: #545454; }
.ansi48-102 { background: #545454; }
.ansi38-103 { color: #54547e; }
.ansi48-103 { background: #54547e; }
.ansi38-104 { color: #5454a8; }
.ansi48-104 { background: #5454a8; }
.ansi38-105 { color: #5454d2; }
.ansi48-105 { background: #5454d2; }
.ansi38-136 { color: #7e5400; }
.ansi48-136 { background: #7e5400; }
.ansi38-137 { color: #7e542a; }
.ansi48-137 { background: #7e542a; }
.ansi38-138 { color: #7e5454; }
.ansi48-138 { background: #7e5454; }
.ansi38-139 { color: #7e547e; }
.ansi48-139 { background: #7e547e; }
.ansi38-140 { color: #7e54a8; }
.ansi48-140 { background: #7e54a8; }
.ansi38-141 { color: #7e54d2; }
.ansi48-141 { background: #7e54d2; }
.ansi38-172 { color: #a85400; }
.ansi48-172 { background: #a85400; }
.ansi38-173 { color: #a8542a; }
.ansi48-173 { background: #a8542a; }
.ansi38-174 { color: #a85454; }
.ansi48-174 { background: #a85454; }
.ansi38-175 { color: #a8547e; }
.ansi48-175 { background: #a8547e; }
.ansi38-176 { color: #a854a8; }
.ansi48-176 { background: #a854a8; }
.ansi38-177 { color: #a854d2; }
.ansi48-177 { background: #a854d2; }
.ansi38-208 { color: #d25400; }
.ansi48-208 { background: #d25400; }
.ansi38-209 { color: #d2542a; }
.ansi48-209 { background: #d2542a; }
.ansi38-210 { color: #d25454; }
.ansi48-210 { background: #d25454; }
.ansi38-211 { color: #d2547e; }
.ansi48-211 { background: #d2547e; }
.ansi38-212 { color: #d254a8; }
.ansi48-212 { background: #d254a8; }
.ansi38-213 { color: #d254d2; }
.ansi48-213 { background: #d254d2; }
.ansi38-34 { color: #007e00; }
.ansi48-34 { background: #007e00; }
.ansi38-35 { color: #007e2a; }
.ansi48-35 { background: #007e2a; }
.ansi38-36 { color: #007e54; }
.ansi48-36 { background: #007e54; }
.ansi38-37 { color: #007e7e; }
.ansi48-37 { background: #007e7e; }
.ansi38-38 { color: #007ea8; }
.ansi48-38 { background: #007ea8; }
.ansi38-39 { color: #007ed2; }
.ansi48-39 { background: #007ed2; }
.ansi38-70 { color: #2a7e00; }
.ansi48-70 { background: #2a7e00; }
.ansi38-71 { color: #2a7e2a; }
.ansi48-71 { background: #2a7e2a; }
.ansi38-72 { color: #2a7e54; }
.ansi48-72 { background: #2a7e54; }
.ansi38-73 { color: #2a7e7e; }
.ansi48-73 { background: #2a7e7e; }
.ansi38-74 { color: #2a7ea8; }
.ansi48-74 { background: #2a7ea8; }
.ansi38-75 { color: #2a7ed2; }
.ansi48-75 { background: #2a7ed2; }
.ansi38-106 { color: #547e00; }
.ansi48-106 { background: #547e00; }
.ansi38-107 { color: #547e2a; }
.ansi48-107 { background: #547e2a; }
.ansi38-108 { color: #547e54; }
.ansi48-108 { background: #547e54; }
.ansi38-109 { color: #547e7e; }
.ansi48-109 { background: #547e7e; }
.ansi38-110 { color: #547ea8; }
.ansi48-110 { background: #547ea8; }
.ansi38-111 { color: #547ed2; }
.ansi48-111 { background: #547ed2; }
.ansi38-142 { color: #7e7e00; }
.ansi48-142 { background: #7e7e00; }
.ansi38-143 { color: #7e7e2a; }
.ansi48-143 { background: #7e7e2a; }
.ansi38-144 { color: #7e7e54; }
.ansi48-144 { background: #7e7e54; }
.ansi38-145 { color: #7e7e7e; }
.ansi48-145 { background: #7e7e7e; }
.ansi38-146 { color: #7e7ea8; }
.ansi48-146 { background: #7e7ea8; }
.ansi38-147 { color: #7e7ed2; }
.ansi48-147 { background: #7e7ed2; }
.ansi38-178 { color: #a87e00; }
.ansi48-178 { background: #a87e00; }
.ansi38-179 { color: #a87e2a; }
.ansi48-179 { background: #a87e2a; }
.ansi38-180 { color: #a87e54; }
.ansi48-180 { background: #a87e54; }
.ansi38-181 { color: #a87e7e; }
.ansi48-181 { background: #a87e7e; }
.ansi38-182 { color: #a87ea8; }
.ansi48-182 { background: #a87ea8; }
.ansi38-183 { color: #a87ed2; }
.ansi48-183 { background: #a87ed2; }
.ansi38-214 { color: #d27e00; }
.ansi48-214 { background: #d27e00; }
.ansi38-215 { color: #d27e2a; }
.ansi48-215 { background: #d27e2a; }
.ansi38-216 { color: #d27e54; }
.ansi48-216 { background: #d27e54; }
.ansi38-217 { color: #d27e7e; }
.ansi48-217 { background: #d27e7e; }
.ansi38-218 { color: #d27ea8; }
.ansi48-218 { background: #d27ea8; }
.ansi38-219 { color: #d27ed2; }
.ansi48-219 { background: #d27ed2; }
.ansi38-40 { color: #00a800; }
.ansi48-40 { background: #00a800; }
.ansi38-41 { color: #00a82a; }
.ansi48-41 { background: #00a82a; }
.ansi38-42 { color: #00a854; }
.ansi48-42 { background: #00a854; }
.ansi38-43 { color: #00a87e; }
.ansi48-43 { background: #00a87e; }
.ansi38-44 { color: #00a8a8; }
.ansi48-44 { background: #00a8a8; }
.ansi38-45 { color: #00a8d2; }
.ansi48-45 { background: #00a8d2; }
.ansi38-76 { color: #2aa800; }
.ansi48-76 { background: #2aa800; }
.ansi38-77 { color: #2aa82a; }
.ansi48-77 { background: #2aa82a; }
.ansi38-78 { color: #2aa854; }
.ansi48-78 { background: #2aa854; }
.ansi38-79 { color: #2aa87e; }
.ansi48-79 { background: #2aa87e; }
.ansi38-80 { color: #2aa8a8; }
.ansi48-80 { background: #2aa8a8; }
.ansi38-81 { color: #2aa8d2; }
.ansi48-81 { background: #2aa8d2; }
.ansi38-112 { color: #54a800; }
.ansi48-112 { background: #54a800; }
.ansi38-113 { color: #54a82a; }
.ansi48-113 { background: #54a82a; }
.ansi38-114 { color: #54a854; }
.ansi48-114 { background: #54a854; }
.ansi38-115 { color: #54a87e; }
.ansi48-115 { background: #54a87e; }
.ansi38-116 { color: #54a8a8; }
.ansi48-116 { background: #54a8a8; }
.ansi38-117 { color: #54a8d2; }
.ansi48-117 { background: #54a8d2; }
.ansi38-148 { color: #7ea800; }
.ansi48-148 { background: #7ea800; }
.ansi38-149 { color: #7ea82a; }
.ansi48-149 { background: #7ea82a; }
.ansi38-150 { color: #7ea854; }
.ansi48-150 { background: #7ea854; }
.ansi38-151 { color: #7ea87e; }
.ansi48-151 { background: #7ea87e; }
.ansi38-152 { color: #7ea8a8; }
.ansi48-152 { background: #7ea8a8; }
.ansi38-153 { color: #7ea8d2; }
.ansi48-153 { background: #7ea8d2; }
.ansi38-184 { color: #a8a800; }
.ansi48-184 { background: #a8a800; }
.ansi38-185 { color: #a8a82a; }
.ansi48-185 { background: #a8a82a; }
.ansi38-186 { color: #a8a854; }
.ansi48-186 { background: #a8a854; }
.ansi38-187 { color: #a8a87e; }
.ansi48-187 { background: #a8a87e; }
.ansi38-188 { color: #a8a8a8; }
.ansi48-188 { background: #a8a8a8; }
.ansi38-189 { color: #a8a8d2; }
.ansi48-189 { background: #a8a8d2; }
.ansi38-220 { color: #d2a800; }
.ansi48-220 { background: #d2a800; }
.ansi38-221 { color: #d2a82a; }
.ansi48-221 { background: #d2a82a; }
.ansi38-222 { color: #d2a854; }
.ansi48-222 { background: #d2a854; }
.ansi38-223 { color: #d2a87e; }
.ansi48-223 { background: #d2a87e; }
.ansi38-224 { color: #d2a8a8; }
.ansi48-224 { background: #d2a8a8; }
.ansi38-225 { color: #d2a8d2; }
.ansi48-225 { background: #d2a8d2; }
.ansi38-46 { color: #00d200; }
.ansi48-46 { background: #00d200; }
.ansi38-47 { color: #00d22a; }
.ansi48-47 { background: #00d22a; }
.ansi38-48 { color: #00d254; }
.ansi48-48 { background: #00d254; }
.ansi38-49 { color: #00d27e; }
.ansi48-49 { background: #00d27e; }
.ansi38-50 { color: #00d2a8; }
.ansi48-50 { background: #00d2a8; }
.ansi38-51 { color: #00d2d2; }
.ansi48-51 { background: #00d2d2; }
.ansi38-82 { color: #2ad200; }
.ansi48-82 { background: #2ad200; }
.ansi38-83 { color: #2ad22a; }
.ansi48-83 { background: #2ad22a; }
.ansi38-84 { color: #2ad254; }
.ansi48-84 { background: #2ad254; }
.ansi38-85 { color: #2ad27e; }
.ansi48-85 { background: #2ad27e; }
.ansi38-86 { color: #2ad2a8; }
.ansi48-86 { background: #2ad2a8; }
.ansi38-87 { color: #2ad2d2; }
.ansi48-87 { background: #2ad2d2; }
.ansi38-118 { color: #54d200; }
.ansi48-118 { background: #54d200; }
.ansi38-119 { color: #54d22a; }
.ansi48-119 { background: #54d22a; }
.ansi38-120 { color: #54d254; }
.ansi48-120 { background: #54d254; }
.ansi38-121 { color: #54d27e; }
.ansi48-121 { background: #54d27e; }
.ansi38-122 { color: #54d2a8; }
.ansi48-122 { background: #54d2a8; }
.ansi38-123 { color: #54d2d2; }
.ansi48-123 { background: #54d2d2; }
.ansi38-154 { color: #7ed200; }
.ansi48-154 { background: #7ed200; }
.ansi38-155 { color: #7ed22a; }
.ansi48-155 { background: #7ed22a; }
.ansi38-156 { color: #7ed254; }
.ansi48-156 { background: #7ed254; }
.ansi38-157 { color: #7ed27e; }
.ansi48-157 { background: #7ed27e; }
.ansi38-158 { color: #7ed2a8; }
.ansi48-158 { background: #7ed2a8; }
.ansi38-159 { color: #7ed2d2; }
.ansi48-159 { background: #7ed2d2; }
.ansi38-190 { color: #a8d200; }
.ansi48-190 { background: #a8d200; }
.ansi38-191 { color: #a8d22a; }
.ansi48-191 { background: #a8d22a; }
.ansi38-192 { color: #a8d254; }
.ansi48-192 { background: #a8d254; }
.ansi38-193 { color: #a8d27e; }
.ansi48-193 { background: #a8d27e; }
.ansi38-194 { color: #a8d2a8; }
.ansi48-194 { background: #a8d2a8; }
.ansi38-195 { color: #a8d2d2; }
.ansi48-195 { background: #a8d2d2; }
.ansi38-226 { color: #d2d200; }
.ansi48-226 { background: #d2d200; }
.ansi38-227 { color: #d2d22a; }
.ansi48-227 { background: #d2d22a; }
.ansi38-228 { color: #d2d254; }
.ansi48-228 { background: #d2d254; }
.ansi38-229 { color: #d2d27e; }
.ansi48-229 { background: #d2d27e; }
.ansi38-230 { color: #d2d2a8; }
.ansi48-230 { background: #d2d2a8; }
.ansi38-231 { color: #d2d2d2; }
.ansi48-231 { background: #d2d2d2; }
.ansi38-232 { color: #080808; }
.ansi48-232 { background: #080808; }
.ansi38-233 { color: #121212; }
.ansi48-233 { background: #121212; }
.ansi38-234 { color: #1c1c1c; }
.ansi48-234 { background: #1c1c1c; }
.ansi38-235 { color: #262626; }
.ansi48-235 { background: #262626; }
.ansi38-236 { color: #303030; }
.ansi48-236 { background: #303030; }
.ansi38-237 { color: #3a3a3a; }
.ansi48-237 { background: #3a3a3a; }
.ansi38-238 { color: #444444; }
.ansi48-238 { background: #444444; }
.ansi38-239 { color: #4e4e4e; }
.ansi48-239 { background: #4e4e4e; }
.ansi38-240 { color: #585858; }
.ansi48-240 { background: #585858; }
.ansi38-241 { color: #626262; }
.ansi48-241 { background: #626262; }
.ansi38-242 { color: #6c6c6c; }
.ansi48-242 { background: #6c6c6c; }
.ansi38-243 { color: #767676; }
.ansi48-243 { background: #767676; }
.ansi38-244 { color: #808080; }
.ansi48-244 { background: #808080; }
.ansi38-245 { color: #8a8a8a; }
.ansi48-245 { background: #8a8a8a; }
.ansi38-246 { color: #949494; }
.ansi48-246 { background: #949494; }
.ansi38-247 { color: #9e9e9e; }
.ansi48-247 { background: #9e9e9e; }
.ansi38-248 { color: #a8a8a8; }
.ansi48-248 { background: #a8a8a8; }
.ansi38-249 { color: #b2b2b2; }
.ansi48-249 { background: #b2b2b2; }
.ansi38-250 { color: #bcbcbc; }
.ansi48-250 { background: #bcbcbc; }
.ansi38-251 { color: #c6c6c6; }
.ansi48-251 { background: #c6c6c6; }
.ansi38-252 { color: #d0d0d0; }
.ansi48-252 { background: #d0d0d0; }
.ansi38-253 { color: #dadada; }
.ansi48-253 { background: #dadada; }
.ansi38-254 { color: #e4e4e4; }
.ansi48-254 { background: #e4e4e4; }
.ansi38-255 { color: #eeeeee; }
.ansi48-255 { background: #eeeeee; }
