<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');

/*
 * Use this file to override any settings from config.php
 *
 * For descriptions of the options please refer to config.php.
 */

# URL to the application
$config['base_url'] = '';

// set this to a 32char random string
$config['encryption_key'] = '';

$config['upload_path'] = FCPATH.'data/uploads';

$config['index_page'] = 'index.php';

$config['cache_backend'] = "dummy";
